#include <SPIFFS.h>
#include <ESP32Servo.h>

#define STEP_PIN 12       // GPIO connected to PUL+ (Pulse)
#define DIR_PIN 14        // GPIO connected to DIR+ (Direction)

Servo myservo; // Create a Servo object to control the servo motor
int angleStep = 2; // Step size for servo movement (adjust for speed control)

void setup() {
  myservo.attach(18); // Attach the servo to pin 18
  myservo.write(180);  // Set servo to initial position
  delay(1000);

  Serial.begin(9600);
  pinMode(DIR_PIN, OUTPUT);  // Set DIR pin as output
  pinMode(STEP_PIN, OUTPUT); // Set STEP pin as output

  if (!SPIFFS.begin(true)) {
    Serial.println("SPIFFS Mount Failed");
    return;
  }

  Serial.println("SPIFFS Mounted Successfully!");
  processFile(); // Process the file to move the motor 
}

void loop() {
  // Empty loop since the movements are handled in processFile()
}

// Function to read the file and process the movements
void processFile() {
  File file = SPIFFS.open("/pins.txt", "r");
  if (!file) {
    Serial.println("Failed to open file");
    return;
  }

  while (file.available()) {
    String line = file.readStringUntil('\n'); // Read a line
    line.trim(); // Remove trailing whitespace, tabs, or newlines

    if (line.length() > 0) {
      int fromPos = 0, toPos = 0;

      // Parse "from" and "to" values, skipping invalid lines
      if (sscanf(line.c_str(), "%d %d", &fromPos, &toPos) == 2) {
        int steps = 12 * (toPos - fromPos); // Calculate steps based on position

        // Handle steps greater than 2400 or less than -2400
        if (steps > 1200) {
          digitalWrite(DIR_PIN, LOW);  // Counterclockwise
          delay(2);
          rotateSteps(2400 - steps);  // Shorter path
        } else if (steps < -1200) {
          digitalWrite(DIR_PIN, HIGH);  // Clockwise
          delay(2);
          rotateSteps(2400 + steps);  // Shorter path
        } else {
          // Normal movement
          if (steps > 0) {
            digitalWrite(DIR_PIN, HIGH); // Clockwise
            delay(2);
          } else {
            digitalWrite(DIR_PIN, LOW);  // Counterclockwise
            delay(2);
          }
          rotateSteps(abs(steps));  // Rotate steps as usual
        }

        delay(500);

        // Feeder moves back
        for (int angle = 180; angle >= 20; angle -= angleStep) {
          myservo.write(angle);
          delay(15);
        }

        delay(100);
        // Perform one step clockwise
        singleStepClockwise(12);

        // Feeder moves forward
        for (int angle = 20; angle <= 180; angle += angleStep) {
          myservo.write(angle);
          delay(15);
        }

        delay(100);
        // Perform one step counterclockwise
        singleStepCounterClockwise(12);
        delay(500);
      } else {
        Serial.println("Invalid line format: " + line);
      }
    }
  }
  file.close();
}

// Function to rotate the motor for a given number of steps
void rotateSteps(int steps) {
  for (int i = 0; i < steps; i++) { 
    digitalWrite(STEP_PIN, HIGH); // Step pulse HIGH
    delayMicroseconds(3000);      // Pulse width (adjust for motor speed)
    digitalWrite(STEP_PIN, LOW);  // Step pulse LOW
    delayMicroseconds(3000);      // Step delay (adjust for motor speed)
  }
  delay(1000); 
}

// Function to perform a single step clockwise
void singleStepClockwise(int clockstep) {
  for (int i = 0; i < clockstep; i++) {
    digitalWrite(DIR_PIN, HIGH); // Set direction to clockwise
    delayMicroseconds(15);       // Allow direction pin to settle
    digitalWrite(STEP_PIN, HIGH); // Step pulse HIGH
    delayMicroseconds(5000);      // Pulse width
    digitalWrite(STEP_PIN, LOW);  // Step pulse LOW
    delayMicroseconds(5000);      // Step delay        
  }
}

// Function to perform a single step counterclockwise
void singleStepCounterClockwise(int counterstep) {
  for (int i = 0; i < counterstep; i++) {
    digitalWrite(DIR_PIN, LOW); // Set direction to counterclockwise
    delayMicroseconds(15);      // Allow direction pin to settle
    digitalWrite(STEP_PIN, HIGH); // Step pulse HIGH
    delayMicroseconds(5000);      // Pulse width
    digitalWrite(STEP_PIN, LOW);  // Step pulse LOW
    delayMicroseconds(5000);      // Step delay   
  } 
}
 
